# -*- coding: utf-8 -*-
"""
Created on Fri Jun  3 14:38:33 2016

@author: mzhong
"""
import pandas as pd
import csv
import numpy as np
from pandas import HDFStore
from sklearn.cross_validation import train_test_split
from scipy.io import wavfile
import random
import json
import glob
import scipy.io

import sys
import os
import time
import theano
import theano.tensor as T
import lasagne

### Some global variables
windowlength = 1 # in seconds
frequency = 318 # sampling rate
nosOfTestSamples = 500
 
# ##################### Build the neural network model #######################
# This script defines a
# function that takes a Theano variable representing the input and returns
# the output layer of a neural network model built in Lasagne.   

def build_cnn_mnist(input_var=None, dropout=False):
    # As a third model, we'll create a CNN of two convolution + pooling stages
    # and a fully-connected hidden layer in front of the output layer.
    if dropout:
        p = 0.5
    else:
        p = 0.0

    # Input layer, as usual:
    network = lasagne.layers.InputLayer(shape=(None, 1, 1, frequency),
                                        input_var=input_var)
    # This time we do not apply input dropout, as it tends to work less well
    # for convolutional layers.
    #print(network.output_shape)                                    
    # Convolutional layer with 32 kernels of size 5x5. Strided and padded
    # convolutions are supported as well; see the docstring.
    network = lasagne.layers.Conv2DLayer(
            network, num_filters=20, filter_size=(1, 5),
            nonlinearity=lasagne.nonlinearities.rectify,
            W=lasagne.init.GlorotUniform())
    # Expert note: Lasagne provides alternative convolutional layers that
    # override Theano's choice of which implementation to use; for details
    # please see http://lasagne.readthedocs.org/en/latest/user/tutorial.html.
    #print(network.output_shape)
    # Max-pooling layer of factor 2 in both dimensions:
    #network = lasagne.layers.MaxPool2DLayer(network, pool_size=(1, 2))
    #print(network.output_shape)
    # Another convolution with 32 5x5 kernels, and another 2x2 pooling:
    network = lasagne.layers.Conv2DLayer(
            network, num_filters=20, filter_size=(1, 5),
            nonlinearity=lasagne.nonlinearities.rectify)
    #network = lasagne.layers.MaxPool2DLayer(network, pool_size=(2, 2))
    #print(network.output_shape)
    # A fully-connected layer of 256 units with 50% dropout on its inputs:
    network = lasagne.layers.DenseLayer(
            lasagne.layers.dropout(network, p=p),
            num_units=frequency,
            nonlinearity=lasagne.nonlinearities.rectify)
    #print(network.output_shape)
    
    network = lasagne.layers.DenseLayer(
            lasagne.layers.dropout(network, p=p),
            num_units=frequency,
            nonlinearity=lasagne.nonlinearities.rectify)
    #print(network.output_shape)
    
#    network = lasagne.layers.DenseLayer(
#            lasagne.layers.dropout(network, p=p),
#            num_units=frequency,
#            nonlinearity=lasagne.nonlinearities.rectify)
#    print(network.output_shape)
    # And, finally, the 10-unit output layer with 50% dropout on its inputs:
#    network = lasagne.layers.DenseLayer(
#            lasagne.layers.dropout(network, p=.5),
#            num_units=2,
#            nonlinearity=lasagne.nonlinearities.softmax)
            
    network = lasagne.layers.DenseLayer(
            lasagne.layers.dropout(network, p=p),
            num_units=2,
            nonlinearity=lasagne.nonlinearities.softmax)
    #print(network.output_shape)
    return network
               
# ############################# Batch iterator ###############################
# This is just a simple helper function iterating over training data in
# mini-batches of a particular size, optionally in random order. It assumes
# data is available as numpy arrays. For big datasets, you could load numpy
# arrays as memory-mapped files (np.load(..., mmap_mode='r')), or write your
# own custom data iteration function. For small datasets, you can also copy
# them to GPU at once for slightly improved performance. This would involve
# several changes in the main program, though, and is not demonstrated here.

        
# ############################## Main program ################################
# Everything else will be handled in our main program now. We could pull out
# more functions to better separate the code, but it wouldn't make it any
# easier to read.

def prediction(model='cnn_mnist',txtfile='feature.txt'):
    # load the test data for prediction
    #test_data = read_valheartwave()
    crs = open(txtfile, "r")
    reading = []
    for columns in ( raw.strip().split() for raw in crs ):  
        reading.append(columns[0])
    reading = np.array(reading).astype('float32')
    
    # load the trained model for prediction
    # Prepare Theano variables for inputs and targets
    input_var = T.tensor4('input')
    #target_var = T.ivector('targets')
    # Create neural network model (depending on first command line parameter)
    #print("Building model and compiling functions...")
    network = build_cnn_mnist(input_var,dropout=False)
        
    with np.load('heart_feature_model.npz') as f:
        param_values = [f['arr_%d' % i] for i in range(len(f.files))]
    lasagne.layers.set_all_param_values(network, param_values)
    
    # make the prediction
    output = lasagne.layers.get_output(network)
    pred_class = T.argmax(output, axis=1)
    test_fn = theano.function([input_var],pred_class)
    inputs = reading.astype('float32')
    inputs = np.reshape(inputs,(1,1,1,len(inputs)))
    predicted_class_label = test_fn(inputs)
    
    return 2*predicted_class_label[0]-1
    
